package ogloszenia.soap;

import jakarta.jws.WebParam;
import jakarta.jws.WebResult;
import jakarta.jws.WebService;
import ogloszenia.Ustawienia;
import ogloszenia.baza.sqlite.DostepDoBazySqlite;
import ogloszenia.baza.sqlite.OgloszenieDAO;
import ogloszenia.exn.BladBazyDanych;
import ogloszenia.exn.NieznanyRekord;
import ogloszenia.model.OgloszenieSamochodowe;

import java.io.IOException;
import java.math.BigDecimal;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

@WebService
public class Ogloszenia {

    @WebResult(name="ogloszenie")
    public List<OgloszenieSamochodowe> readAll() throws BladBazyDanych {
        try (DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            OgloszenieDAO dao = db.newOgloszenieDAO();
            return dao.readAllFull();
        }
    }

    @WebResult(name="ogloszenie")
    public OgloszenieSamochodowe readOne(@WebParam(name="id") int idOgloszenia) throws BladBazyDanych, NieznanyRekord {
        try (DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            OgloszenieDAO dao = db.newOgloszenieDAO();
            return dao.findByIdFull(idOgloszenia);
        }
    }

    @WebResult(name="ogloszenie")
    public List<OgloszenieSamochodowe> readByPrice(
            @WebParam(name="min") BigDecimal min,
            @WebParam(name="max") BigDecimal max) throws BladBazyDanych {
        try(DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            OgloszenieDAO dao = db.newOgloszenieDAO();
            return dao.ogloszeniaWedlugCeny(min, max);
        }
    }

    public void update(
            @WebParam(name="ogloszenie") OgloszenieSamochodowe ogloszenie) throws BladBazyDanych, NieznanyRekord {
        // W tej wersji zakładamy, że obiekt ma podane id i nadpisujemy istniejące.
        // W tym przypadku nie trzeba dawać informacji zwrotnej.
        try(DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            db.beginTransaction();
            OgloszenieDAO dao = db.newOgloszenieDAO();
            boolean sukces = dao.update(ogloszenie);
            if(!sukces) {
                throw new NieznanyRekord("Brak ogłoszenia nr " + ogloszenie.getIdOgloszenia());
            }
            db.endTransaction(true);
        }
    }

    @WebResult(name="new-id")
    public int insert(
            @WebParam(name="ogloszenie") OgloszenieSamochodowe ogloszenie) throws BladBazyDanych {
        // W tej wersji zawsze dodajemy nowy rekord.
        // W razie, gdyby klient przysłał ogłoszenie z ustawionym id, to my i tak to id zignorujemy.
        // W tej wersji w odpowiedzi wysyłamy id nowego rekordu, aby klient dowiedział się jakie ono jest.
        ogloszenie.setIdOgloszenia(null);
        try(DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            db.beginTransaction();
            OgloszenieDAO dao = db.newOgloszenieDAO();
            dao.insertNew(ogloszenie);
            db.endTransaction(true);
            return ogloszenie.getIdOgloszenia();
        }
    }

    @WebResult(name="bytes")
    public byte[] photo(@WebParam(name="id") int idOgloszenia) throws NieznanyRekord {
        try {
            Path path = Paths.get(Ustawienia.PHOTOS_PATH, idOgloszenia + ".jpg");
            return Files.readAllBytes(path);
        } catch (IOException e) {
            throw new NieznanyRekord("Brak zdjęcia dla ogłoszenia nr " + idOgloszenia);
        }
    }
}
